import json
import os

from jupyter_server.base.handlers import APIHandler
from jupyter_server.utils import url_path_join
import tornado

from .salome_server.handler import SalomeHandler

API_NAMESPACE = "jupytercad-salome"


class RouteHandler(APIHandler):
    @tornado.web.authenticated
    def get(self):
        """Return the base url of the jupytercad salome server.
        Use ``None`` if it is the same JupyterLab server
        """
        base_url_env = os.getenv("SALOME_SERVER_BASE_URL", None)
        self.finish(
            json.dumps(
                {
                    "backend_url": base_url_env,
                }
            )
        )


def setup_handlers(web_app):
    host_pattern = ".*$"

    base_url = web_app.settings["base_url"]

    route_pattern = url_path_join(base_url, API_NAMESPACE, "get-backend")

    salome_pattern = url_path_join(base_url, API_NAMESPACE, "execute")

    handlers = [(route_pattern, RouteHandler), (salome_pattern, SalomeHandler)]

    web_app.add_handlers(host_pattern, handlers)



from pathlib import Path
import salome
import os
import base64
from tempfile import NamedTemporaryFile
from salome.geom import geomBuilder
from salome.smesh import smeshBuilder

salome.salome_init_without_session()
geompy = geomBuilder.New()
smesh = smeshBuilder.New()


def build_mesh(brep_string: str, number_of_segment: int, jcad_path: str) -> str:
    if number_of_segment < 0:
        raise ValueError("Number of segment negative")

    content = ""
    brep_temp = NamedTemporaryFile(delete=False)
    with open(brep_temp.name, "w") as f:
        f.write(brep_string)
        f.seek(0)

    geometry = geompy.ImportBREP(brep_temp.name)
    mesh = smesh.Mesh(geometry)

    hypo = mesh.Tetrahedron(algo=smeshBuilder.NETGEN_1D2D3D)
    params = hypo.Parameters(smeshBuilder.SIMPLE)
    params.SetNumberOfSegments(number_of_segment)
    params.LengthFromEdges()
    params.LengthFromFaces()

    is_done = mesh.Compute()
    if not is_done:
        raise RuntimeError("Problem during mesh computation")
    if jcad_path:
        med_path = Path.cwd() / Path(jcad_path.replace(".jcad", ".med"))
        mesh.ExportMED(str(med_path), 1)
    stl_tmp = NamedTemporaryFile(suffix=".stl", delete=False)

    mesh.ExportSTL(stl_tmp.name, 0)  # O means exporting in binary format
    stl_tmp.seek(0)

    with open(stl_tmp.name, "rb") as f_obj:
        content = f_obj.read()
    try:
        os.remove(stl_tmp.name)
        os.remove(brep_temp.name)
    except Exception:
        pass
    base64_content = base64.b64encode(content).decode("utf8")
    return base64_content



import json
from jupyter_server.base.handlers import APIHandler
import tornado

try:
    from .mesh_builder import build_mesh
except ImportError:
    build_mesh = None


class SalomeHandler(APIHandler):
    @tornado.web.authenticated
    def post(self):
        if build_mesh is None:
            self.finish(
                json.dumps(
                    {
                        "mesh": "",
                        "format": "obj",
                        "error": "Can not import Salome, please check if Salome is correctly installed",
                    }
                )
            )
        else:
            data = self.get_json_body()
            geometry = data.get("geometry", None)
            n_segmen = data.get("numberOfSegments", 12)
            jcad_path: str = data.get("sourcePath", None)
            if jcad_path:
                jcad_path = jcad_path.replace("RTC:", "")

            mesh_content = ""
            if geometry:
                mesh_content = build_mesh(geometry, n_segmen, jcad_path)

            self.finish(json.dumps({"mesh": mesh_content, "format": "obj"}))
